@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# (c) 2009-2021 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
+# (c) 2009-2022 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
 # Original PyFileServer (c) 2005 Ho Chun Wei.
 # Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
 """
@@ -75,13 +75,11 @@ Testability and caveats
    This class is being tested for a network domain (I'm setting one up to test).
 
 """
-from __future__ import print_function
-
 import win32net
 import win32netcon
 import win32security
 
-from wsgidav import compat, util
+from wsgidav import util
 from wsgidav.dc.base_dc import BaseDomainController
 
 __docformat__ = "reStructuredText"
@@ -90,7 +88,7 @@ _logger = util.get_module_logger(__name__)
 
 class NTDomainController(BaseDomainController):
     def __init__(self, wsgidav_app, config):
-        super(NTDomainController, self).__init__(wsgidav_app, config)
+        super().__init__(wsgidav_app, config)
         # auth_conf = config["http_authenticator"]
         dc_conf = config.get("nt_dc", {})
 
@@ -175,11 +173,11 @@ class NTDomainController(BaseDomainController):
                     server, 0, win32netcon.FILTER_NORMAL_ACCOUNT, 0
                 )
                 # Make sure, we compare unicode
-                un = compat.to_unicode(user_name).lower()
+                un = util.to_str(user_name).lower()
                 for userinfo in users:
                     uiname = userinfo.get("name")
                     assert uiname
-                    assert compat.is_unicode(uiname)
+                    assert util.is_str(uiname)
                     if un == userinfo["name"].lower():
                         return True
             except win32net.error as e:
