@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# (c) 2009-2021 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
+# (c) 2009-2022 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
 # Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
 """
 Implementation of a domain controller that allows users to authenticate against
@@ -9,8 +9,6 @@ Used by HTTPAuthenticator. Only available on linux and macOS.
 
 See https://wsgidav.readthedocs.io/en/latest/user_guide_configure.html
 """
-from __future__ import print_function
-
 import pam
 
 from wsgidav import util
@@ -22,7 +20,7 @@ _logger = util.get_module_logger(__name__)
 
 class PAMDomainController(BaseDomainController):
     def __init__(self, wsgidav_app, config):
-        super(PAMDomainController, self).__init__(wsgidav_app, config)
+        super().__init__(wsgidav_app, config)
 
         self.pam = pam.pam()
 
