@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# (c) 2009-2021 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
+# (c) 2009-2022 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
 # Original PyFileServer (c) 2005 Ho Chun Wei.
 # Licensed under the MIT license:
 # http://www.opensource.org/licenses/mit-license.php
@@ -18,7 +18,7 @@ from wsgidav.dav_error import (
     as_DAVError,
     get_http_status_string,
 )
-from wsgidav.middleware import BaseMiddleware
+from wsgidav.mw.base_mw import BaseMiddleware
 
 __docformat__ = "reStructuredText"
 
@@ -30,9 +30,8 @@ _logger = util.get_module_logger(__name__)
 # ========================================================================
 class ErrorPrinter(BaseMiddleware):
     def __init__(self, wsgidav_app, next_app, config):
-        super(ErrorPrinter, self).__init__(wsgidav_app, next_app, config)
+        super().__init__(wsgidav_app, next_app, config)
         self.err_config = config.get("error_printer", {})
-        self.catch_all_exceptions = self.err_config.get("catch_all", False)
 
     def is_disabled(self):
         return self.err_config.get("enable") is False
@@ -75,24 +74,15 @@ class ErrorPrinter(BaseMiddleware):
                     )
 
                 return
-            except DAVError as e:
-                _logger.debug("re-raising {}".format(e))
-                raise
+            except DAVError:
+                raise  # Deliberately generated or already converted
             except Exception as e:
                 # Caught a non-DAVError
-                if self.catch_all_exceptions:
-                    # Catch all exceptions to return as 500 Internal Error
-                    # traceback.print_exc(10, environ.get("wsgi.errors") or sys.stderr)
-                    _logger.error("{}".format(traceback.format_exc(10)))
-                    raise as_DAVError(e)
-                else:
-                    _logger.error(
-                        "Caught Exception\n{}".format(traceback.format_exc(10))
-                    )
-                    # traceback.print_exc(10, sys.stderr)
-                    raise
+                # Catch all exceptions to return as 500 Internal Error
+                _logger.error("{}".format(traceback.format_exc(10)))
+                raise as_DAVError(e)
         except DAVError as e:
-            _logger.debug("caught {}".format(e))
+            _logger.debug("Caught {}".format(e))
 
             status = get_http_status_string(e)
             # Dump internal errors to console
