@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# (c) 2009-2021 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
+# (c) 2009-2022 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
 # Original PyFileServer (c) 2005 Ho Chun Wei.
 # Licensed under the MIT license:
 # http://www.opensource.org/licenses/mit-license.php
@@ -77,6 +77,7 @@ The environ variable here is the WSGI 'environ' dictionary. It is passed to
 all methods of the domain controller as a means for developers to pass information
 from previous middleware or server config (if required).
 """
+import base64
 import inspect
 import random
 import re
@@ -84,9 +85,10 @@ import time
 from hashlib import md5
 from textwrap import dedent
 
-from wsgidav import compat, util
+from wsgidav import util
+from wsgidav.dav_error import HTTP_NOT_FOUND, DAVError
 from wsgidav.dc.simple_dc import SimpleDomainController
-from wsgidav.middleware import BaseMiddleware
+from wsgidav.mw.base_mw import BaseMiddleware
 from wsgidav.util import calc_base64, calc_hexdigest, dynamic_import_class
 
 __docformat__ = "reStructuredText"
@@ -100,7 +102,7 @@ def make_domain_controller(wsgidav_app, config):
     if dc is True or not dc:
         # True or null:
         dc = SimpleDomainController
-    elif compat.is_basestring(dc):
+    elif util.is_basestring(dc):
         # If a plain string is passed, try to import it as class
         dc = dynamic_import_class(dc)
 
@@ -133,7 +135,7 @@ class HTTPAuthenticator(BaseMiddleware):
     )
 
     def __init__(self, wsgidav_app, next_app, config):
-        super(HTTPAuthenticator, self).__init__(wsgidav_app, next_app, config)
+        super().__init__(wsgidav_app, next_app, config)
         self._verbose = config.get("verbose", 3)
         self.config = config
 
@@ -275,7 +277,7 @@ class HTTPAuthenticator(BaseMiddleware):
         _logger.debug("401 Not Authorized for realm '{}' (basic)".format(realm))
         wwwauthheaders = 'Basic realm="{}"'.format(realm)
 
-        body = compat.to_bytes(self.error_message_401)
+        body = util.to_bytes(self.error_message_401)
         start_response(
             "401 Not Authorized",
             [
@@ -296,8 +298,8 @@ class HTTPAuthenticator(BaseMiddleware):
         except Exception:
             auth_value = ""
 
-        auth_value = compat.base64_decodebytes(compat.to_bytes(auth_value))
-        auth_value = compat.to_native(auth_value)
+        auth_value = base64.decodebytes(util.to_bytes(auth_value))
+        auth_value = util.to_str(auth_value)
         user_name, password = auth_value.split(":", 1)
 
         if self.domain_controller.basic_auth_user(realm, user_name, password, environ):
@@ -334,7 +336,7 @@ class HTTPAuthenticator(BaseMiddleware):
             )
         )
 
-        body = compat.to_bytes(self.error_message_401)
+        body = util.to_bytes(self.error_message_401)
         start_response(
             "401 Not Authorized",
             [
@@ -350,6 +352,12 @@ class HTTPAuthenticator(BaseMiddleware):
 
         realm = self.domain_controller.get_domain_realm(environ["PATH_INFO"], environ)
 
+        if not realm:
+            raise DAVError(
+                HTTP_NOT_FOUND,
+                context_info=f"Could not resolve realm for {environ['PATH_INFO']}",
+            )
+
         is_invalid_req = False
         invalid_req_reasons = []
 
@@ -484,7 +492,7 @@ class HTTPAuthenticator(BaseMiddleware):
         if not is_invalid_req:
             req_method = environ["REQUEST_METHOD"]
 
-            required_digest = self.compute_digest_response(
+            required_digest = self._compute_digest_response(
                 realm,
                 req_username,
                 req_method,
@@ -506,14 +514,14 @@ class HTTPAuthenticator(BaseMiddleware):
                 )
             elif required_digest != req_response:
                 warning_msg = (
-                    "compute_digest_response('{}', '{}', ...): {} != {}".format(
+                    "_compute_digest_response('{}', '{}', ...): {} != {}".format(
                         realm, req_username, required_digest, req_response
                     )
                 )
                 if self.winxp_accept_root_share_login and realm != "/":
                     # _logger.warning(warning_msg + " => trying '/' realm")
                     # Hotfix: also accept '/' digest
-                    root_digest = self.compute_digest_response(
+                    root_digest = self._compute_digest_response(
                         "/",
                         req_username,
                         req_method,
@@ -563,7 +571,7 @@ class HTTPAuthenticator(BaseMiddleware):
         environ["wsgidav.auth.user_name"] = req_username
         return self.next_app(environ, start_response)
 
-    def compute_digest_response(
+    def _compute_digest_response(
         self, realm, user_name, method, uri, nonce, cnonce, qop, nc, environ
     ):
         """Computes digest hash.
@@ -586,7 +594,7 @@ class HTTPAuthenticator(BaseMiddleware):
         """
 
         def md5h(data):
-            return md5(compat.to_bytes(data)).hexdigest()
+            return md5(util.to_bytes(data)).hexdigest()
 
         def md5kd(secret, data):
             return md5h(secret + ":" + data)
