@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# (c) 2009-2021 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
+# (c) 2009-2022 Martin Wendt and contributors; see WsgiDAV https://github.com/mar10/wsgidav
 # Original PyFileServer (c) 2005 Ho Chun Wei.
 # Licensed under the MIT license:
 # http://www.opensource.org/licenses/mit-license.php
@@ -52,19 +52,20 @@ import inspect
 import platform
 import sys
 import time
+from urllib.parse import unquote
 
-from wsgidav import __version__, compat, util
+from wsgidav import __version__, util
 from wsgidav.dav_provider import DAVProvider
 from wsgidav.default_conf import DEFAULT_CONFIG
 from wsgidav.fs_dav_provider import FilesystemProvider
 from wsgidav.http_authenticator import HTTPAuthenticator
-from wsgidav.lock_manager import LockManager
-from wsgidav.lock_storage import LockStorageDict
-from wsgidav.middleware import BaseMiddleware
+from wsgidav.lock_man.lock_manager import LockManager
+from wsgidav.lock_man.lock_storage import LockStorageDict
+from wsgidav.mw.base_mw import BaseMiddleware
 from wsgidav.prop_man.property_manager import PropertyManager
 from wsgidav.util import (
     dynamic_import_class,
-    dynamic_instantiate_middleware,
+    dynamic_instantiate_class_from_opts,
     safe_re_encode,
 )
 
@@ -85,28 +86,38 @@ def _check_config(config):
         "acceptbasic": "http_authenticator.accept_basic",
         "acceptdigest": "http_authenticator.accept_digest",
         "catchall": "error_printer.catch_all",
+        "debug_litmus": "logging.debug_litmus",
+        "debug_methods": "logging.debug_methods",
         "defaultdigest": "http_authenticator.default_to_digest",
         "dir_browser.app_class": "middleware_stack",
-        # "dir_browser.enable": "middleware_stack",
+        "dir_browser.ms_mount": "(removed)",
         "dir_browser.ms_sharepoint_plugin": "dir_browser.ms_sharepoint_support",
         "dir_browser.ms_sharepoint_url": "dir_browser.ms_sharepoint_support",
         "domain_controller": "http_authenticator.domain_controller",
         "domaincontroller": "http_authenticator.domain_controller",
-        "emulate_win32_lastmod": "hotfix.emulate_win32_lastmod",
+        "emulate_win32_lastmod": "hotfixes.emulate_win32_lastmod",
+        "enable_loggers": "logging.enable_loggers",
+        "error_printer.catch_all": "(removed)",
         "http_authenticator.preset_domain": "nt_dc.preset_domain",
         "http_authenticator.preset_server": "nt_dc.preset_server",
         "locksmanager": "lock_manager",
+        "lock_manager": "lock_storage",
+        "logger_date_format": "logging.logger_date_format",
+        "logger_format": "logging.logger_format",
+        "logging.verbose": "verbose",  # prevent a likely mistake
         "mutableLiveProps": "mutable_live_props",
         "propsmanager": "property_manager",
-        "re_encode_path_info": "hotfix.re_encode_path_info",
+        "re_encode_path_info": "hotfixes.re_encode_path_info",
+        "response_headers": "(see Cors middleware)",
         "trusted_auth_header": "http_authenticator.trusted_auth_header",
-        "unquote_path_info": "hotfix.unquote_path_info",
+        "unquote_path_info": "hotfixes.unquote_path_info",
         "user_mapping": "simple_dc.user_mapping",
+        # "dir_browser.enable": "middleware_stack",
     }
     for old, new in deprecated_fields.items():
         if "." in old:
             k, v = old.split(".", 1)
-            d = config[k]
+            d = config.get(k, {})
         else:
             d, v = config, old
 
@@ -122,13 +133,15 @@ def _check_config(config):
 # ========================================================================
 # WsgiDAVApp
 # ========================================================================
-class WsgiDAVApp(object):
+class WsgiDAVApp:
     def __init__(self, config):
 
         self.config = copy.deepcopy(DEFAULT_CONFIG)
         util.deep_update(self.config, config)
         config = self.config
 
+        expand = {"${application}": self}
+
         # Evaluate configuration and set defaults
         _check_config(config)
 
@@ -136,27 +149,40 @@ class WsgiDAVApp(object):
 
         hotfixes = config.get("hotfixes", {})
 
-        self.re_encode_path_info = hotfixes.get("re_encode_path_info", None)
-        if self.re_encode_path_info is None:
-            self.re_encode_path_info = compat.PY3
-
+        self.re_encode_path_info = hotfixes.get("re_encode_path_info", True)
+        if type(self.re_encode_path_info) is not bool:
+            raise ValueError("re_encode_path_info must be bool (or omitted)")
         self.unquote_path_info = hotfixes.get("unquote_path_info", False)
 
-        lock_storage = config.get("lock_manager")
+        lock_storage = config.get("lock_storage")
         if lock_storage is True:
             lock_storage = LockStorageDict()
+        elif isinstance(lock_storage, (str, dict)):
+            lock_storage = dynamic_instantiate_class_from_opts(
+                lock_storage, expand=expand
+            )
 
         if not lock_storage:
+            # Normalize False, 0 to None
             self.lock_manager = None
         else:
+            if not hasattr(lock_storage, "refresh"):
+                raise ValueError(f"Invalid lock_storage: {lock_storage!r}")
             self.lock_manager = LockManager(lock_storage)
 
-        self.prop_manager = config.get("property_manager")
-        if not self.prop_manager:
+        prop_manager = config.get("property_manager")
+        if prop_manager is True:
+            prop_manager = PropertyManager()
+        elif isinstance(prop_manager, (str, dict)):
+            prop_manager = dynamic_instantiate_class_from_opts(
+                prop_manager, expand=expand
+            )
+
+        if not prop_manager:
             # Normalize False, 0 to None
             self.prop_manager = None
-        elif self.prop_manager is True:
-            self.prop_manager = PropertyManager()
+        else:
+            self.prop_manager = prop_manager
 
         self.mount_path = config.get("mount_path")
         auth_conf = config.get("http_authenticator", {})
@@ -195,23 +221,19 @@ class WsgiDAVApp(object):
             # The middleware stack configuration may contain plain strings, dicts,
             # classes, or objects
             app = None
-            if compat.is_basestring(mw):
+            if util.is_basestring(mw):
                 # If a plain string is passed, try to import it, assuming
                 # `BaseMiddleware` signature
                 app_class = dynamic_import_class(mw)
                 app = app_class(self, self.application, config)
             elif type(mw) is dict:
-                # If a dict with one entry is passed, use the key as module/class name
-                # and the value as constructor arguments (positional or kwargs).
-                assert len(mw) == 1
-                name, args = list(mw.items())[0]
+                # If a dict with one entry is passed, expect {class: ..., kwargs: ...}
                 expand = {"${application}": self.application}
-                app = dynamic_instantiate_middleware(name, args, expand)
+                app = dynamic_instantiate_class_from_opts(mw, expand=expand)
             elif inspect.isclass(mw):
                 # If a class is passed, assume BaseMiddleware (or compatible)
-                assert issubclass(
-                    mw, BaseMiddleware
-                )  # TODO: remove this assert with 3.0
+                # TODO: remove this assert with 3.0
+                assert issubclass(mw, BaseMiddleware)
                 app = mw(self, self.application, config)
             else:
                 # Otherwise assume an initialized middleware instance
@@ -234,8 +256,6 @@ class WsgiDAVApp(object):
             else:
                 _logger.error("Could not add middleware {}.".format(mw))
 
-        domain_controller
-        # Print info
         _logger.info(
             "WsgiDAV/{} Python/{} {}".format(
                 __version__, util.PYTHON_VERSION, platform.platform(aliased=True)
@@ -288,40 +308,45 @@ class WsgiDAVApp(object):
                     )
         return
 
-    def add_provider(self, share, provider, readonly=False):
+    def add_provider(self, share, provider, *, readonly=False):
         """Add a provider to the provider_map routing table."""
         # Make sure share starts with, or is '/'
         share = "/" + share.strip("/")
         assert share not in self.provider_map
 
-        if compat.is_basestring(provider):
+        if type(provider) is str:
             # Syntax:
             #   <mount_path>: <folder_path>
             # We allow a simple string as 'provider'. In this case we interpret
             # it as a file system root folder that is published.
-            provider = FilesystemProvider(provider, readonly)
+            provider = util.fix_path(provider, self.config)
+            provider = FilesystemProvider(provider, readonly=readonly)
+
         elif type(provider) in (dict,):
-            if "provider" in provider:
+            if "class" in provider:
                 # Syntax:
-                #   <mount_path>: {"provider": <class_path>, "args": <pos_args>, "kwargs": <named_args}
-                prov_class = dynamic_import_class(provider["provider"])
-                provider = prov_class(
-                    *provider.get("args", []), **provider.get("kwargs", {})
-                )
-            else:
+                #   <mount_path>: {"class": <class_path>, "args": <pos_args>, "kwargs": <named_args>}
+                expand = {"${application}": self}
+                provider = dynamic_instantiate_class_from_opts(provider, expand=expand)
+            elif "root" in provider:
                 # Syntax:
                 #   <mount_path>: {"root": <path>, "redaonly": <bool>}
                 provider = FilesystemProvider(
-                    provider["root"], bool(provider.get("readonly", False))
+                    util.fix_path(provider["root"], self.config),
+                    readonly=bool(provider.get("readonly", False)),
+                )
+            else:
+                raise ValueError(
+                    f"Provider expected {{'class': ...}}` or {{'root': ...}}: {provider}"
                 )
         elif type(provider) in (list, tuple):
             raise ValueError(
-                "Provider {}: tuple/list syntax is no longer supported".format(provider)
+                f"Provider {provider}: tuple/list syntax is no longer supported"
             )
             # provider = FilesystemProvider(provider[0], provider[1])
 
         if not isinstance(provider, DAVProvider):
-            raise ValueError("Invalid provider {}".format(provider))
+            raise ValueError(f"Invalid provider {provider}")
 
         provider.set_share_path(share)
         if self.mount_path:
@@ -372,27 +397,26 @@ class WsgiDAVApp(object):
 
         path = environ["PATH_INFO"]
 
-        # (#73) Failed on processing non-iso-8859-1 characters on Python 3
-        #
-        # Note: we encode using UTF-8 here (falling back to ISO-8859-1)!
-        # This seems to be wrong, since per PEP 3333 PATH_INFO is always ISO-8859-1 encoded
-        # (see https://www.python.org/dev/peps/pep-3333/#unicode-issues).
-        # But also seems to resolve errors when accessing resources with Chinese characters, for
-        # example.
-        # This is done by default for Python 3, but can be turned off in settings.
+        # WSGI always assumes iso-8859-1. Modern clients send UTF-8, so we may
+        # have to re-encode.
+        # See also:
+        # - Issue #73
+        # - https://www.python.org/dev/peps/pep-3333/#unicode-issues
+        # - https://bugs.python.org/issue16679#msg177450
+        # (The hotfixes.re_encode_path_info option is true by default.)
         if self.re_encode_path_info:
-            path = environ["PATH_INFO"] = compat.wsgi_to_bytes(path).decode()
+            path = environ["PATH_INFO"] = util.re_encode_wsgi(path)
 
         # We optionally unquote PATH_INFO here, although this should already be
-        # done by the server (#8).
+        # done by the server (#8, #228).
         if self.unquote_path_info:
-            path = compat.unquote(environ["PATH_INFO"])
+            path = unquote(environ["PATH_INFO"])
 
         # GC issue 22: Pylons sends root as u'/'
-        if not compat.is_native(path):
-            _logger.warning("Got non-native PATH_INFO: {!r}".format(path))
+        if not util.is_str(path):
+            _logger.warning(f"Got non-native PATH_INFO: {path!r}")
             # path = path.encode("utf8")
-            path = compat.to_native(path)
+            path = util.to_str(path)
 
         # Always adding these values to environ:
         environ["wsgidav.config"] = self.config
@@ -433,7 +457,7 @@ class WsgiDAVApp(object):
             environ["PATH_INFO"] = path[len(share) :]
 
         # assert isinstance(path, str)
-        assert compat.is_native(path)
+        assert util.is_str(path)
         # See http://mail.python.org/pipermail/web-sig/2007-January/002475.html
         # for some clarification about SCRIPT_NAME/PATH_INFO format
         # SCRIPT_NAME starts with '/' or is empty
@@ -452,7 +476,7 @@ class WsgiDAVApp(object):
             headerDict = {}
             for header, value in response_headers:
                 if header.lower() in headerDict:
-                    _logger.error("Duplicate header in response: {}".format(header))
+                    _logger.error(f"Duplicate header in response: {header}")
                 headerDict[header.lower()] = value
 
             # Check if we should close the connection after this request.
@@ -538,8 +562,8 @@ class WsgiDAVApp(object):
                     extra.append("elap={:.3f}sec".format(time.time() - start_time))
                 extra = ", ".join(extra)
 
-                #               This is the CherryPy format:
-                #                127.0.0.1 - - [08/Jul/2009:17:25:23] "GET /loginPrompt?redirect=/renderActionList%3Frelation%3Dpersonal%26key%3D%26filter%3DprivateSchedule&reason=0 HTTP/1.1" 200 1944 "http://127.0.0.1:8002/command?id=CMD_Schedule" "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.1) Gecko/20090624 Firefox/3.5"  # noqa
+                # This is the CherryPy format:
+                #   127.0.0.1 - - [08/Jul/2009:17:25:23] "GET /loginPrompt?redirect=/renderActionList%3Frelation%3Dpersonal%26key%3D%26filter%3DprivateSchedule&reason=0 HTTP/1.1" 200 1944 "http://127.0.0.1:8002/command?id=CMD_Schedule" "Mozilla/5.0 (Windows; U; Windows NT 6.0; de; rv:1.9.1) Gecko/20090624 Firefox/3.5"  # noqa
                 _logger.info(
                     '{addr} - {user} - [{time}] "{method} {path}" {extra} -> {status}'.format(
                         addr=environ.get("REMOTE_ADDR", ""),
